@@ -5,11 +5,11 @@
 
 :Author:
     David Young
-
-:Date Created:
-    December 19, 2016
 """
-################# GLOBAL IMPORTS ####################
+from __future__ import division
+from builtins import zip
+from builtins import object
+from past.utils import old_div
 import sys
 import os
 import math
@@ -19,41 +19,44 @@ from sherlock.catalogue_conesearch import catalogue_conesearch
 from astrocalc.distances import converter
 from astrocalc.coords import separations
 import time
+from fundamentals.mysql import database
 
-
-class transient_catalogue_crossmatch():
+class transient_catalogue_crossmatch(object):
     """
     *crossmatch a list of transients against a suite of catalogues according to given search algorithm*
 
-    **Key Arguments:**
-        - ``dbConn`` -- mysql database connection for the catalogues
-        - ``log`` -- logger
-        - ``settings`` -- the settings dictionary
-        - ``colMaps`` -- maps of the important column names for each table/view in the crossmatch-catalogues database
-        - ``transients`` -- the list of transients
+    **Key Arguments**
 
-    **Usage:**
+    - ``dbConn`` -- mysql database connection for the catalogues
+    - ``log`` -- logger
+    - ``settings`` -- the settings dictionary
+    - ``colMaps`` -- maps of the important column names for each table/view in the crossmatch-catalogues database
+    - ``transients`` -- the list of transients
+    
 
-        To setup your logger, settings and database connections, please use the ``fundamentals`` package (`see tutorial here <http://fundamentals.readthedocs.io/en/latest/#tutorial>`_).
+    **Usage**
 
-        To initiate a transient_catalogue_crossmatch object, use the following:
+    To setup your logger, settings and database connections, please use the ``fundamentals`` package (`see tutorial here <http://fundamentals.readthedocs.io/en/latest/#tutorial>`_).
 
-        .. code-block:: python
+    To initiate a transient_catalogue_crossmatch object, use the following:
 
-            from sherlock import transient_catalogue_crossmatch
-            xmatcher = transient_catalogue_crossmatch(
-                log=log,
-                settings=settings,
-                dbConn=dbConn,
-                colMaps=colMaps,
-                transients=transients
-            )
+    ```python
+    from sherlock import transient_catalogue_crossmatch
+    xmatcher = transient_catalogue_crossmatch(
+        log=log,
+        settings=settings,
+        colMaps=colMaps,
+        transients=transients,
+        dbSettings=settings["database settings"]["static catalogues"]
+    )
+    ```
 
-        Then to run the transient through the search algorithm found in the settings file, use the ``match`` method:
+    Then to run the transient through the search algorithm found in the settings file, use the ``match`` method:
 
-        .. code-block:: python
-
-            classifications = xmatcher.match()
+    ```python
+    classifications = xmatcher.match()
+    ```
+    
 
     .. todo ::
 
@@ -70,10 +73,11 @@ class transient_catalogue_crossmatch():
     def __init__(
             self,
             log,
-            dbConn=False,
             settings=False,
             colMaps=False,
-            transients=[]
+            transients=[],
+            dbSettings=False,
+            dbConn=False
     ):
         self.log = log
         log.debug("instansiating a new 'transient_catalogue_crossmatch' object")
@@ -82,6 +86,12 @@ class transient_catalogue_crossmatch():
         self.transients = transients
         self.colMaps = colMaps
 
+        if dbSettings:
+            self.dbConn = database(
+                log=self.log,
+                dbSettings=dbSettings
+            ).connect()
+
         # xt-self-arg-tmpx
         return None
 
@@ -89,8 +99,10 @@ class transient_catalogue_crossmatch():
         """
         *match the transients against the sherlock-catalogues according to the search algorithm and return matches alongside the predicted classification(s)*
 
-        **Return:**
-            - ``classification`` -- the crossmatch results and classifications assigned to the transients
+        **Return**
+
+        - ``classification`` -- the crossmatch results and classifications assigned to the transients
+        
 
         See the class docstring for usage.
 
@@ -124,7 +136,7 @@ class transient_catalogue_crossmatch():
         # ITERATE THROUGH SEARCH ALGORITHM IN ORDER
         # PRESENTED IN THE SETTINGS FILE
         brightnessFilters = ["bright", "faint", "general"]
-        for search_name, searchPara in sa.iteritems():
+        for search_name, searchPara in list(sa.items()):
             for bf in brightnessFilters:
                 if bf not in searchPara:
                     continue
@@ -171,7 +183,7 @@ class transient_catalogue_crossmatch():
         # ITERATE THROUGH SEARCH ALGORITHM IN ORDER
         # PRESENTED IN THE SETTINGS FILE
         if len(nonSynonymTransients) > 0:
-            for search_name, searchPara in sa.iteritems():
+            for search_name, searchPara in list(sa.items()):
                 self.log.debug("""  searching: %(search_name)s""" % locals())
                 for bf in brightnessFilters:
                     if bf not in searchPara:
@@ -221,7 +233,7 @@ class transient_catalogue_crossmatch():
         # ITERATE THROUGH SEARCH ALGORITHM IN ORDER
         # PRESENTED IN THE SETTINGS FILE
         brightnessFilters = ["bright", "faint", "general"]
-        for search_name, searchPara in sa.iteritems():
+        for search_name, searchPara in list(sa.items()):
             for bf in brightnessFilters:
                 if bf not in searchPara:
                     continue
@@ -272,71 +284,75 @@ class transient_catalogue_crossmatch():
     ):
         """*perform an angular separation crossmatch against a given catalogue in the database and annotate the crossmatch with some value added parameters (distances, physical separations, sub-type of transient etc)*
 
-        **Key Arguments:**
-            - ``objectList`` -- the list of transient locations to match against the crossmatch catalogue
-            - ``searchPara`` -- the search parameters for this individual search as lifted from the search algorithm in the sherlock settings file
-            - ``search_name`` -- the name of the search as given in the sherlock settings file
-            - ``brightnessFilter`` -- is this search to be constrained by magnitude of the catalogue sources? Default *False*. [bright|faint|general]
-            - ``physicalSearch`` -- is this angular search a sub-part of a physical separation search
-            - ``classificationType`` -- synonym, association or annotation. Default *False*
+        **Key Arguments**
 
-         **Return:**
+        - ``objectList`` -- the list of transient locations to match against the crossmatch catalogue
+        - ``searchPara`` -- the search parameters for this individual search as lifted from the search algorithm in the sherlock settings file
+        - ``search_name`` -- the name of the search as given in the sherlock settings file
+        - ``brightnessFilter`` -- is this search to be constrained by magnitude of the catalogue sources? Default *False*. [bright|faint|general]
+        - ``physicalSearch`` -- is this angular search a sub-part of a physical separation search
+        - ``classificationType`` -- synonym, association or annotation. Default *False*
+        
+
+         **Return**
+
+         
             - matchedObjects -- any sources matched against the object
 
-        **Usage:**
+        **Usage**
 
-            Take a list of transients from somewhere
+        Take a list of transients from somewhere
 
-            .. code-block:: python 
+        ```python
+        transients = [
+            {'ps1_designation': u'PS1-14aef',
+             'name': u'4L3Piiq',
+             'detection_list_id': 2,
+             'local_comments': u'',
+             'ra': 0.02548233704918263,
+             'followup_id': 2065412L,
+             'dec': -4.284933417540423,
+             'id': 1000006110041705700L,
+             'object_classification': 0L
+             },
 
+            {'ps1_designation': u'PS1-13dcr',
+             'name': u'3I3Phzx',
+             'detection_list_id': 2,
+             'local_comments': u'',
+             'ra': 4.754236999477372,
+             'followup_id': 1140386L,
+             'dec': 28.276703631398625,
+             'id': 1001901011281636100L,
+             'object_classification': 0L
+             },
 
-                transients = [
-                    {'ps1_designation': u'PS1-14aef',
-                     'name': u'4L3Piiq',
-                     'detection_list_id': 2,
-                     'local_comments': u'',
-                     'ra': 0.02548233704918263,
-                     'followup_id': 2065412L,
-                     'dec': -4.284933417540423,
-                     'id': 1000006110041705700L,
-                     'object_classification': 0L
-                     },
+            {'ps1_designation': u'PS1-13dhc',
+             'name': u'3I3Pixd',
+             'detection_list_id': 2,
+             'local_comments': u'',
+             'ra': 1.3324973428505413,
+             'followup_id': 1202386L,
+             'dec': 32.98869220595689,
+             'id': 1000519791325919200L,
+             'object_classification': 0L
+             }
+        ]
+        ```
 
-                    {'ps1_designation': u'PS1-13dcr',
-                     'name': u'3I3Phzx',
-                     'detection_list_id': 2,
-                     'local_comments': u'',
-                     'ra': 4.754236999477372,
-                     'followup_id': 1140386L,
-                     'dec': 28.276703631398625,
-                     'id': 1001901011281636100L,
-                     'object_classification': 0L
-                     },
+        Then run the ``angular_crossmatch_against_catalogue`` method to crossmatch against the catalogues and return results:
 
-                    {'ps1_designation': u'PS1-13dhc',
-                     'name': u'3I3Pixd',
-                     'detection_list_id': 2,
-                     'local_comments': u'',
-                     'ra': 1.3324973428505413,
-                     'followup_id': 1202386L,
-                     'dec': 32.98869220595689,
-                     'id': 1000519791325919200L,
-                     'object_classification': 0L
-                     }
-                ]
-
-            Then run the ``angular_crossmatch_against_catalogue`` method to crossmatch against the catalogues and return results:
-
-            .. code-block:: python 
-
-                # ANGULAR CONESEARCH ON CATALOGUE
-                search_name = "ned_d spec sn"
-                searchPara = self.settings["search algorithm"][search_name]
-                matchedObjects = xmatcher.angular_crossmatch_against_catalogue(
-                    objectList=transients,
-                    searchPara=searchPara,
-                    search_name=search_name
-                )
+        ```python
+        # ANGULAR CONESEARCH ON CATALOGUE
+        search_name = "ned_d spec sn"
+        searchPara = self.settings["search algorithm"][search_name]
+        matchedObjects = xmatcher.angular_crossmatch_against_catalogue(
+            objectList=transients,
+            searchPara=searchPara,
+            search_name=search_name
+        )
+        ```
+        
 
         .. todo ::
 
@@ -519,14 +535,18 @@ class transient_catalogue_crossmatch():
             search_name):
         """*annotate each crossmatch with physical parameters such are distances etc*
 
-        **Key Arguments:**
-            - ``crossmatchDict`` -- the crossmatch dictionary
-            - ``catalogueName`` -- the name of the catalogue the crossmatch results from
-            - ``searchPara`` -- the search parameters for this individual search as lifted from the search algorithm in the sherlock settings file
-            - ``search_name`` -- the name of the search as given in the sherlock settings file
+        **Key Arguments**
 
-        **Return:**
-            - ``crossmatchDict`` -- the annotated crossmatch dictionary
+        - ``crossmatchDict`` -- the crossmatch dictionary
+        - ``catalogueName`` -- the name of the catalogue the crossmatch results from
+        - ``searchPara`` -- the search parameters for this individual search as lifted from the search algorithm in the sherlock settings file
+        - ``search_name`` -- the name of the search as given in the sherlock settings file
+        
+
+        **Return**
+
+        - ``crossmatchDict`` -- the annotated crossmatch dictionary
+        
 
         .. todo ::
 
@@ -582,7 +602,7 @@ class transient_catalogue_crossmatch():
         # ADD DISTANCE VALUES
         if "distance" in crossmatchDict and crossmatchDict["distance"]:
             direct_distance = crossmatchDict["distance"]
-            direct_distance_scale = direct_distance / 206.264806
+            direct_distance_scale = old_div(direct_distance, 206.264806)
             direct_distance_modulus = 5 * \
                 math.log10(direct_distance * 1e6) - 5
         # crossmatchDict['z'] = z
@@ -628,14 +648,18 @@ class transient_catalogue_crossmatch():
             lowerMagnitudeLimit):
         """*perform a bright star match on the crossmatch results if required by the catalogue search*
 
-        **Key Arguments:**
-            - ``matchedObjects`` -- the list of matched sources from the catalogue crossmatch
-            - ``catalogueName`` -- the name of the catalogue the crossmatch results from
-            - ``magnitudeLimitFilter`` -- the name of the column containing the magnitude to filter on
-            - ``lowerMagnitudeLimit`` -- the lower magnitude limit to match bright stars against
+        **Key Arguments**
 
-        **Return:**
-            - ``brightStarMatches`` -- the trimmed matched sources (bright stars associations only)
+        - ``matchedObjects`` -- the list of matched sources from the catalogue crossmatch
+        - ``catalogueName`` -- the name of the catalogue the crossmatch results from
+        - ``magnitudeLimitFilter`` -- the name of the column containing the magnitude to filter on
+        - ``lowerMagnitudeLimit`` -- the lower magnitude limit to match bright stars against
+        
+
+        **Return**
+
+        - ``brightStarMatches`` -- the trimmed matched sources (bright stars associations only)
+        
 
         .. todo ::
 
@@ -673,15 +697,19 @@ class transient_catalogue_crossmatch():
             lowerMagnitudeLimit):
         """*perform a bright star match on the crossmatch results if required by the catalogue search*
 
-        **Key Arguments:**
-            - ``matchedObjects`` -- the list of matched sources from the catalogue crossmatch
-            - ``catalogueName`` -- the name of the catalogue the crossmatch results from
-            - ``magnitudeLimitFilter`` -- the name of the column containing the magnitude to filter on
-            - ``lowerMagnitudeLimit`` -- the lower magnitude limit to match general galaxies against
-            - ``upperMagnitudeLimit`` -- the upper magnitude limit to match general galaxies against
+        **Key Arguments**
 
-        **Return:**
-            - ``galaxyMatches`` -- the trimmed matched sources (associated galaxies only)
+        - ``matchedObjects`` -- the list of matched sources from the catalogue crossmatch
+        - ``catalogueName`` -- the name of the catalogue the crossmatch results from
+        - ``magnitudeLimitFilter`` -- the name of the column containing the magnitude to filter on
+        - ``lowerMagnitudeLimit`` -- the lower magnitude limit to match general galaxies against
+        - ``upperMagnitudeLimit`` -- the upper magnitude limit to match general galaxies against
+        
+
+        **Return**
+
+        - ``galaxyMatches`` -- the trimmed matched sources (associated galaxies only)
+        
 
         .. todo ::
 
@@ -707,7 +735,7 @@ class transient_catalogue_crossmatch():
                 mag = decimal.Decimal(row[magnitudeLimitFilter])
                 if mag and mag < lowerMagnitudeLimit and mag > upperMagnitudeLimit:
                     sep = decimal.Decimal(row["separationArcsec"])
-                    if sep < decimal.Decimal(decimal.Decimal(10)**(decimal.Decimal((decimal.Decimal(25.) - mag) / decimal.Decimal(6.)))):
+                    if sep < decimal.Decimal(decimal.Decimal(10)**(decimal.Decimal(old_div((decimal.Decimal(25.) - mag), decimal.Decimal(6.))))):
                         galaxyMatches.append(row)
 
         self.log.debug('completed the ``_galaxy_association_cuts`` method')
@@ -725,29 +753,34 @@ class transient_catalogue_crossmatch():
 
         This search is basically the same as the angular separation search except extra filtering is done to exclude sources outside the physical search radius (matched sources require distance info to calulate physical separations)
 
-        **Key Arguments:**
-            - ``objectList`` -- transients to be crossmatched
-            - ``searchPara`` -- parameters of the search (from settings file)
-            - ``search_name`` -- the name of the search
-            - ``brightnessFilter`` -- is this search to be constrained by magnitude of the catalogue sources? Default *False*. [bright|faint|general]
-            - ``classificationType`` -- synonym, association or annotation. Default *False*
+        **Key Arguments**
 
-        **Return:**
-            - matchedObjects -- any sources matched against the object
+        - ``objectList`` -- transients to be crossmatched
+        - ``searchPara`` -- parameters of the search (from settings file)
+        - ``search_name`` -- the name of the search
+        - ``brightnessFilter`` -- is this search to be constrained by magnitude of the catalogue sources? Default *False*. [bright|faint|general]
+        - ``classificationType`` -- synonym, association or annotation. Default *False*
+        
+
+        **Return**
+
+        - matchedObjects -- any sources matched against the object
+        
 
         To run a physical separation crossmatch, run in a similar way to the angular separation crossmatch:
 
-        **Usage:**
+        **Usage**
 
-            .. code-block:: python 
-
-                search_name = "ned spec sn"
-                searchPara = self.settings["search algorithm"][search_name]
-                matchedObjects = xmatcher.physical_separation_crossmatch_against_catalogue(
-                    objectList=transients,
-                    searchPara=searchPara,
-                    search_name=search_name
-                )
+        ```python
+        search_name = "ned spec sn"
+        searchPara = self.settings["search algorithm"][search_name]
+        matchedObjects = xmatcher.physical_separation_crossmatch_against_catalogue(
+            objectList=transients,
+            searchPara=searchPara,
+            search_name=search_name
+        )
+        ```
+        
 
         .. todo ::
 
@@ -820,21 +853,20 @@ class transient_catalogue_crossmatch():
                                 self.settings["galaxy radius stetch factor"],)
                         newAngularSep = row[
                             "major_axis_arcsec"] * self.settings["galaxy radius stetch factor"]
-                    else:
-                        continue
+
                 # NOW CHECK FOR A DIRECT DISTANCE MEASUREMENT
-                elif row["direct_distance_scale"] and physical_separation_kpc < physicalRadius:
+                if row["direct_distance_scale"] and physical_separation_kpc < physicalRadius and thisMatch == False:
                     if row["separationArcsec"] > 300.:
                         continue
                     thisMatch = True
                     newsearch_name = newsearch_name + " (direct distance)"
-                    newAngularSep = physicalRadius / \
-                        row["direct_distance_scale"]
+                    newAngularSep = old_div(physicalRadius, \
+                        row["direct_distance_scale"])
                 # NEW CHECK FOR A REDSHIFT DISTANCE
                 elif row["scale"] and physical_separation_kpc < physicalRadius:
                     thisMatch = True
                     newsearch_name = newsearch_name + " (redshift distance)"
-                    newAngularSep = physicalRadius / row["scale"]
+                    newAngularSep = old_div(physicalRadius, row["scale"])
 
                 if thisMatch == True:
                     row["physical_separation_kpc"] = physical_separation_kpc
